<?php
/**
 * Created by PhpStorm.
 * User: Desenv-005
 * Date: 19/12/2018
 * Time: 17:37
 */

namespace Schaffen\WarehouseInventory\WarehouseInventory\Services;

use Schaffen\Core\Product\Models\ProductVarietyPack;
use Schaffen\WarehouseInventory\WarehouseInventory\Models\WarehouseInventory;

class VerifyCanMove
{
    private $pack;
    private $variety;
    private $product;
    private $warehouse_inventory;
    private $item;

    const OUT = 0;
    const IN = 1;

    public function __construct(
       ProductVarietyPack $pack,
       WarehouseInventory $warehouse_inventory,
       $item = [],
       $atualizar_valor_compra = 1,
       $atualizar_valor_medio = 1
    ){

        $return = [];
        $this->pack = $pack;
        $this->variety = $this->pack->variety;
        $this->product = $this->variety->product;
        $this->warehouse_inventory = $warehouse_inventory;
        $this->item = $item;

        $this->validateItem();
        $return['cc'] = $this->geraEntradaNaContaCorrente();
        $return['snapshot'] = $this->atualizarSnapshot($atualizar_valor_medio);
        if ($atualizar_valor_compra) {
            $this->atualizarValorCompra();
        }
        return $return;
    }
   public function validateItem()
   {

      $verifica_estoque_atual = true;
      $verificaEstoqueNegativo = Parametro::getDadosParametro(Parametro::PARAMETRO_ESTOQUE_NEGATIVO);
      if($verificaEstoqueNegativo->par_valor == 1){
         $nao_permite_estoque_negativo = false;
      }else{
         $nao_permite_estoque_negativo = true;
      }

      if ((isset($this->pack->pve_controla_estoque) && $this->pack->pve_controla_estoque == 0 && $this->warehouse_inventory->loe_tipo != LocalEstoque::LOCAL_FUNCIONARIO)) {
         $verifica_estoque_atual = false;
      }

      if (empty($this->pack)) {
         throw new SCHException("O código da pack deve ser informado.");
      }
      if (empty($this->warehouse_inventory)) {
         throw new SCHException("O local de estoque deve ser informado.");
      }


      $validation_rules = [
         'quantidade' => 'required|min:1',
         'sentido' => 'required|in:' . SELF::OUT . ',' . SELF::IN,
         'operacao' => 'required',
         'codigo_operacao' => 'required',
      ];

      if ($this->product->pro_controla_serial == '1') {
         $this->item['serial'] = trim($this->item['serial']);
         $validation_rules = array_merge($validation_rules, ['serial' => 'required']);
      }

      $validator = Validator::make($this->item, $validation_rules);

      if ($validator->fails()) {
         throw new SCHException($validator->messages()->first());
      }

      $snap = Snapshot::where("sna_fk_pve_codigo", $this->pack->pve_codigo)->where('sna_fk_loe_codigo',$this->warehouse_inventory->loe_codigo)->first();
      if(!empty($snap)){
         $snapshot_atual = $this->getEstoqueAtual();
      }else{
         throw new SCHException("Este product não existe no local de estoque deste vendedor.");
      }


      if ($verifica_estoque_atual) {
         $this->verificaEstoqueAtual($snapshot_atual);
      }

      if ($nao_permite_estoque_negativo && $this->item['sentido'] == self::OUT) {

         $quantidade_atual = $snapshot_atual->sna_quantidade;
         $composicao = $this->product->pro_descricao . ' - ' . $this->variety->prv_descricao . "(" . $this->pack->pve_codigo_referencia. ")";

         if ($quantidade_atual - $this->item['quantidade'] < 0) {
            throw new SCHException("A movimentação não pode ser realizada para o product {$composicao} pois o " .
               "estoque ficará negativo. Quantidade atual: {$quantidade_atual}. " .
               "Quantidade movimentada: {$this->item['quantidade']}.");
         }
      }


   }

   protected function verificaEstoqueAtual($snapshot)
   {

      if (empty($snapshot) && $this->item['sentido'] == self::OUT) {
         $composicao = $this->product->pro_descricao || ' - ' || $this->variety->prv_descricao;
         if (isset($this->item['serial']) && !empty($this->item['serial'])) {
            $composicao += " com o serial {$this->item['serial']} ";
         }
         throw new SCHException("Não foi possível localizar o product '{$composicao}' no estoque informado.");
      }

      $estoque_atual = $snapshot->sna_quantidade;

      if ($this->item['sentido'] == self::IN &&
         $this->product->pro_controla_serial == '1') {
         if ($estoque_atual + $this->item['quantidade'] > 1) {
            throw new SCHException("Já foi realizada uma entrada do product {$this->product->nome_completo} com o serial {$this->item['serial']}.");
         }
      }

      return $snapshot;
   }

   public function getEstoqueAtual()
   {
      $snapshot = Snapshot::query();
      $snapshot->where("sna_fk_pve_codigo", $this->pack->pve_codigo);
      //obtém as informações de estoque atual do product
      if (!empty($this->item['serial'])) {
         $snapshot->where("sna_serial", $this->item['serial']);
         if ($this->item['sentido'] == self::OUT) {
            $snapshot->where("sna_fk_loe_codigo", $this->warehouse_inventory->loe_codigo);
         }
      } else {
         $snapshot->where("sna_fk_loe_codigo", $this->warehouse_inventory->loe_codigo);
      }
      $snapshot = $snapshot->first();

      if (!$snapshot) {
         $snapshot = Snapshot::create(
            [
               "sna_fk_pve_codigo" => $this->pack->pve_codigo,
               "sna_fk_loe_codigo" => $this->warehouse_inventory->loe_codigo,
               "sna_valor_medio" => 1,
               "sna_quantidade" => 1
            ]
         );
      }

      return $snapshot;
   }



}
